@@ -34,6 +34,8 @@
 
 /**
  * SECTION:gstappsrc
+ * @short_description: Easy way for applications to inject buffers into a
+ *     pipeline
  * @see_also: #GstBaseSrc, appsink
  *
  * The appsrc element can be used by applications to insert data into a
@@ -836,6 +838,7 @@ gst_app_src_do_seek (GstBaseSrc * src, GstSegment * segment)
   if (res) {
     GST_DEBUG_OBJECT (appsrc, "flushing queue");
     gst_app_src_flush_queued (appsrc);
+    appsrc->priv->is_eos = FALSE;
   } else {
     GST_WARNING_OBJECT (appsrc, "seek failed");
   }
