@@ -91,8 +91,8 @@ enum
 };
 static guint gst_input_selector_signals[LAST_SIGNAL] = { 0 };
 
-static gboolean gst_input_selector_is_active_sinkpad (GstInputSelector * sel,
-    GstPad * pad);
+static inline gboolean gst_input_selector_is_active_sinkpad (GstInputSelector *
+    sel, GstPad * pad);
 static GstPad *gst_input_selector_activate_sinkpad (GstInputSelector * sel,
     GstPad * pad);
 static GstPad *gst_input_selector_get_linked_pad (GstPad * pad,
@@ -148,6 +148,7 @@ static gint64 gst_selector_pad_get_running_time (GstSelectorPad * pad);
 static void gst_selector_pad_reset (GstSelectorPad * pad);
 static gboolean gst_selector_pad_event (GstPad * pad, GstEvent * event);
 static GstCaps *gst_selector_pad_getcaps (GstPad * pad);
+static gboolean gst_selector_pad_acceptcaps (GstPad * pad, GstCaps * caps);
 static GstIterator *gst_selector_pad_iterate_linked_pads (GstPad * pad);
 static GstFlowReturn gst_selector_pad_chain (GstPad * pad, GstBuffer * buf);
 static GstFlowReturn gst_selector_pad_bufferalloc (GstPad * pad,
@@ -189,10 +190,8 @@ gst_selector_pad_class_init (GstSelectorPadClass * klass)
 
   gobject_class->finalize = gst_selector_pad_finalize;
 
-  gobject_class->get_property =
-      GST_DEBUG_FUNCPTR (gst_selector_pad_get_property);
-  gobject_class->set_property =
-      GST_DEBUG_FUNCPTR (gst_selector_pad_set_property);
+  gobject_class->get_property = gst_selector_pad_get_property;
+  gobject_class->set_property = gst_selector_pad_set_property;
 
   g_object_class_install_property (gobject_class, PROP_PAD_RUNNING_TIME,
       g_param_spec_int64 ("running-time", "Running time",
@@ -457,7 +456,7 @@ gst_selector_pad_getcaps (GstPad * pad)
   sel = GST_INPUT_SELECTOR (gst_pad_get_parent (pad));
 
   GST_DEBUG_OBJECT (sel, "Getting caps of srcpad peer");
-  caps = gst_pad_peer_get_caps (sel->srcpad);
+  caps = gst_pad_peer_get_caps_reffed (sel->srcpad);
   if (caps == NULL)
     caps = gst_caps_new_any ();
 
@@ -466,6 +465,21 @@ gst_selector_pad_getcaps (GstPad * pad)
   return caps;
 }
 
+static gboolean
+gst_selector_pad_acceptcaps (GstPad * pad, GstCaps * caps)
+{
+  GstInputSelector *sel;
+  gboolean res;
+
+  sel = GST_INPUT_SELECTOR (gst_pad_get_parent (pad));
+
+  GST_DEBUG_OBJECT (sel, "Checking acceptcaps of srcpad peer");
+  res = gst_pad_peer_accept_caps (sel->srcpad, caps);
+  gst_object_unref (sel);
+
+  return res;
+}
+
 static GstFlowReturn
 gst_selector_pad_bufferalloc (GstPad * pad, guint64 offset,
     guint size, GstCaps * caps, GstBuffer ** buf)
@@ -543,9 +557,10 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   GstPad *active_sinkpad;
   GstPad *prev_active_sinkpad;
   GstSelectorPad *selpad;
-  GstClockTime end_time, duration;
+  GstClockTime start_time;
   GstSegment *seg;
   GstEvent *close_event = NULL, *start_event = NULL;
+  GstCaps *caps;
 
   sel = GST_INPUT_SELECTOR (gst_pad_get_parent (pad));
   selpad = GST_SELECTOR_PAD_CAST (pad);
@@ -562,16 +577,16 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   active_sinkpad = gst_input_selector_activate_sinkpad (sel, pad);
 
   /* update the segment on the srcpad */
-  end_time = GST_BUFFER_TIMESTAMP (buf);
-  if (GST_CLOCK_TIME_IS_VALID (end_time)) {
-    duration = GST_BUFFER_DURATION (buf);
-    if (GST_CLOCK_TIME_IS_VALID (duration))
-      end_time += duration;
-    GST_DEBUG_OBJECT (pad, "received end time %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (end_time));
+  start_time = GST_BUFFER_TIMESTAMP (buf);
+  if (GST_CLOCK_TIME_IS_VALID (start_time)) {
+    GST_DEBUG_OBJECT (pad, "received start time %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (start_time));
+    if (GST_BUFFER_DURATION_IS_VALID (buf))
+      GST_DEBUG_OBJECT (pad, "received end time %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (start_time + GST_BUFFER_DURATION (buf)));
 
     GST_OBJECT_LOCK (pad);
-    gst_segment_set_last_stop (seg, seg->format, end_time);
+    gst_segment_set_last_stop (seg, seg->format, start_time);
     GST_OBJECT_UNLOCK (pad);
   }
 
@@ -583,7 +598,7 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
     GstSegment *cseg = &sel->segment;
 
     GST_DEBUG_OBJECT (sel,
-        "pushing NEWSEGMENT update %d, rate %lf, applied rate %lf, "
+        "pushing close NEWSEGMENT update %d, rate %lf, applied rate %lf, "
         "format %d, "
         "%" G_GINT64_FORMAT " -- %" G_GINT64_FORMAT ", time %"
         G_GINT64_FORMAT, TRUE, cseg->rate, cseg->applied_rate, cseg->format,
@@ -598,7 +613,7 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   /* if we have a pending segment, push it out now */
   if (G_UNLIKELY (selpad->segment_pending)) {
     GST_DEBUG_OBJECT (pad,
-        "pushing NEWSEGMENT update %d, rate %lf, applied rate %lf, "
+        "pushing pending NEWSEGMENT update %d, rate %lf, applied rate %lf, "
         "format %d, "
         "%" G_GINT64_FORMAT " -- %" G_GINT64_FORMAT ", time %"
         G_GINT64_FORMAT, FALSE, seg->rate, seg->applied_rate, seg->format,
@@ -632,6 +647,11 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   GST_DEBUG_OBJECT (pad, "Forwarding buffer %p from pad %s:%s", buf,
       GST_DEBUG_PAD_NAME (pad));
 
+  if ((caps = GST_BUFFER_CAPS (buf))) {
+    if (GST_PAD_CAPS (sel->srcpad) != caps)
+      gst_pad_set_caps (sel->srcpad, caps);
+  }
+
   res = gst_pad_push (sel->srcpad, buf);
 
 done:
@@ -743,10 +763,8 @@ gst_input_selector_class_init (GstInputSelectorClass * klass)
 
   gobject_class->dispose = gst_input_selector_dispose;
 
-  gobject_class->set_property =
-      GST_DEBUG_FUNCPTR (gst_input_selector_set_property);
-  gobject_class->get_property =
-      GST_DEBUG_FUNCPTR (gst_input_selector_get_property);
+  gobject_class->set_property = gst_input_selector_set_property;
+  gobject_class->get_property = gst_input_selector_get_property;
 
   g_object_class_install_property (gobject_class, PROP_N_PADS,
       g_param_spec_uint ("n-pads", "Number of Pads",
@@ -887,7 +905,10 @@ gst_input_selector_dispose (GObject * object)
 static gint64
 gst_segment_get_timestamp (GstSegment * segment, gint64 running_time)
 {
-  return (running_time - segment->accum) * segment->abs_rate + segment->start;
+  if (running_time <= segment->accum)
+    return segment->start;
+  else
+    return (running_time - segment->accum) * segment->abs_rate + segment->start;
 }
 
 static void
@@ -1174,7 +1195,7 @@ gst_input_selector_getcaps (GstPad * pad)
         GST_DEBUG_PAD_NAME (pad), GST_DEBUG_PAD_NAME (otherpad));
     /* if the peer has caps, use those. If the pad is not linked, this function
      * returns NULL and we return ANY */
-    if (!(caps = gst_pad_peer_get_caps (otherpad)))
+    if (!(caps = gst_pad_peer_get_caps_reffed (otherpad)))
       caps = gst_caps_new_any ();
     gst_object_unref (otherpad);
   }
@@ -1245,6 +1266,8 @@ gst_input_selector_request_new_pad (GstElement * element,
       GST_DEBUG_FUNCPTR (gst_selector_pad_event));
   gst_pad_set_getcaps_function (sinkpad,
       GST_DEBUG_FUNCPTR (gst_selector_pad_getcaps));
+  gst_pad_set_acceptcaps_function (sinkpad,
+      GST_DEBUG_FUNCPTR (gst_selector_pad_acceptcaps));
   gst_pad_set_chain_function (sinkpad,
       GST_DEBUG_FUNCPTR (gst_selector_pad_chain));
   gst_pad_set_iterate_internal_links_function (sinkpad,
